//-----------------------------------------------------------
//    mycontroller.cpp
//-----------------------------------------------------------
#include "mycontroller.h"
#include "mouse_stat.h"
#include "../main.h"
#include "../v/myframe.h"
#include "../v/myglcanvas.h"
#include "../s/csound.h"
#include "../m/shapemod/poly_line.h"
#include "../m/shapemod/line.h"
#include "../m/shapemod/dot.h"
#include "../m/wavedata.h"
//-----------------------------------------------------------
//    constructor
//-----------------------------------------------------------
myController::myController(myFrame* f1, cSound* cs1, WaveData *wd1, Poly_Line* pl1)
{
	frame_1 = f1;
	csound_1 = cs1;
	poly_line_1 = pl1;
	wdata_1 = wd1;
	mouse_1 = new Mouse_Status();
}

//------------------------------------------------------------
//    destructor
//------------------------------------------------------------
myController::~myController()
{
}

//-----------------------------------------------------------
//    myController::OnClose
//-----------------------------------------------------------
void myController::OnClose(wxCloseEvent& event)
{
	if(frame_1!=0){
		frame_1->Destroy();
		csound_1->close_pa();
	}
}

//-----------------------------------------------------------
//    myController::OnSlider1
//-----------------------------------------------------------
void myController::OnSlider1(wxScrollEvent& event)
{
	int vol = frame_1->slider_1->GetValue();
	wdata_1->set_amplitude(double(vol) * 0.01);
}

//-----------------------------------------------------------
//    myController::OnKeyCtrl
//-----------------------------------------------------------
void myController::OnKey(wxKeyEvent& event)
{
	int k1 = event.GetKeyCode(), fq1;
	int df = 10, n1;
	bool shif = event.ShiftDown();
	bool ctrl = event.ControlDown();
	if(ctrl) df = 1000;
	else if(shif) df = 100;
	else df = 10;
	switch(k1){
		case WXK_UP:
			fq1 = wdata_1->frequency;
			n1 = fq1 + df;
			if(n1 <= (int)max_freq){
				wdata_1->set_frequency(fq1+df);
				frame_1->text_ctrl_1->Clear();
				frame_1->text_ctrl_1
					->AppendText(wxString::Format(wxT("%d %s"), n1, wxT("Hz")));
			}
			break;
		case WXK_DOWN:
			fq1 = wdata_1->frequency;
			n1 = fq1 - df;
			if(n1 >= (int)min_freq){
				wdata_1->set_frequency(n1);
				frame_1->text_ctrl_1->Clear();
				frame_1->text_ctrl_1
					->AppendText(wxString::Format(wxT("%d %s"), n1, wxT("Hz")));
			}
			break;
	}
}

//-----------------------------------------------------------
//    myController::OnButton
//-----------------------------------------------------------
void myController::OnButton(wxCommandEvent& event)
{
	int b1 = (frame_1->button_1->GetValue());
	int er;
	if(b1) er = csound_1->start_pa();
	else er = csound_1->stop_pa();
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void myController::OnMouseRight(wxMouseEvent& event)
{
	int px = event.m_x, py = event.m_y;
	mouse_1->pos_right_down[0] = px;
	mouse_1->pos_right_down[1] = py;
	int lnum = poly_line_1->select_line(px, canvas_size[1] - py - 1);
	int dnum = poly_line_1->select_dot(px, canvas_size[1] - py - 1);
	int lsz = poly_line_1->get_line_size();
	if((lnum!=-1)||(dnum!=-1)){
		wxMenu *popm = new wxMenu();
		if(lnum!=-1) popm->Append(id_popmenu_select1, wxT("Insert dot"));
		if(dnum!=-1) popm->Append(id_popmenu_select2, wxT("Delete dot"));
		frame_1->myglc_1->PopupMenu(popm, px, py);
		delete popm;
	}
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void myController::OnMouseUP(wxMouseEvent& event)
{
	mouse_1->b_1 = NON;
	mouse_1->select = false;
	mouse_1->count = 0;
}

//-----------------------------------------------------------
//    
//-----------------------------------------------------------
void myController::OnMouseMove(wxMouseEvent& event)
{
	double dr0, dth0, dph0, c1, c2;
	int dx, dy, w1, h1, px, py;
	int x0, y0, xright, xleft;
	bool ctrl, alt;
	static bool select;
	static int num_dot;
	
	if((mouse_1->select)&&(mouse_1->b_1==LEFT)){
		mouse_1->pos[0] = event.m_x;
		mouse_1->pos[1] = event.m_y;
		dx = (mouse_1->pos[0] - mouse_1->pre_pos[0]);
		dy = - (mouse_1->pos[1] - mouse_1->pre_pos[1]);
		
		if((num_dot==0)||(num_dot==(poly_line_1->get_dot_size()-1))){
			dx = 0;
		}else{
			x0 = poly_line_1->dot(num_dot)->get_pos_x();
			xright = poly_line_1->dot(num_dot-1)->get_pos_x();
			xleft = poly_line_1->dot(num_dot+1)->get_pos_x();
			if(((x0 + dx)<xright)||((x0 + dx)>xleft)){
				dx = 0;
			}
		}
		y0 = poly_line_1->dot(num_dot)->get_pos_y();
		if(((y0 + dy)<0)||((y0 + dy)>(canvas_size[1] - 1))){
			dy = 0;
		}
		poly_line_1->move_dot(num_dot, dx, dy);
		frame_1->myglc_1->render();
		mouse_1->pre_pos[0] = mouse_1->pos[0];
		mouse_1->pre_pos[1] = mouse_1->pos[1];
		compose_wave_from_line(wdata_1, poly_line_1);
		mouse_1->count++;
	}
	if(event.LeftDown()){
		mouse_1->b_1 = LEFT;
		mouse_1->pos[0] = event.m_x;
		mouse_1->pos[1] = event.m_y;
		mouse_1->b_1 = LEFT;
		mouse_1->select = poly_line_1->select(mouse_1->pos[0], canvas_size[1] - mouse_1->pos[1]);
		if(mouse_1->select) num_dot = poly_line_1->select_dot(mouse_1->pos[0], canvas_size[1] - mouse_1->pos[1] - 1);
		mouse_1->pre_pos[0] = event.m_x;
		mouse_1->pre_pos[1] = event.m_y;
		mouse_1->count++;
	}
}

//-----------------------------------------------------------
//    myController::OnLeaveCanvas
//-----------------------------------------------------------
void myController::OnLeaveCanvas(wxMouseEvent& event)
{
	mouse_1->b_1 = NON;
	mouse_1->select = false;
	mouse_1->count = 0;
}

//-----------------------------------------------------------
//    myController::OnPaintCanvas
//-----------------------------------------------------------
void myController::OnPaintCanvas(wxPaintEvent& event)
{
	frame_1->myglc_1->render();
}

//-----------------------------------------------------------
//    myController::OnPopMenuSelect1
//-----------------------------------------------------------
void myController::OnPopMenuSelect1(wxCommandEvent& event)
{
	int num_line;
	int xright, xleft;
	int px = mouse_1->pos_right_down[0];
	int py = canvas_size[1] - mouse_1->pos_right_down[1] - 1;
	num_line = poly_line_1->select_line(px, py);
	if(num_line==-1) return;
	xright = poly_line_1->line(num_line)->get_dot_1()->get_pos_x();
	xleft = poly_line_1->line(num_line)->get_dot_2()->get_pos_x();
	if(px<=xright) px = xright;
	if(px>=xleft) px = xleft;
	poly_line_1->insert_dot(px, py);
	compose_wave_from_line(wdata_1, poly_line_1);
	frame_1->myglc_1->render();
}

//-----------------------------------------------------------
//    myController::OnPopMenuSelect2
//-----------------------------------------------------------
void myController::OnPopMenuSelect2(wxCommandEvent& event)
{
	int px = mouse_1->pos_right_down[0];
	int py = canvas_size[1] - mouse_1->pos_right_down[1] - 1;
	int dnum = poly_line_1->select_dot(px, py);
	int dsz = poly_line_1->get_dot_size();
	if((dnum<=0)||(dnum>=dsz-1)) return;
	poly_line_1->erase_dot(dnum);
	compose_wave_from_line(wdata_1, poly_line_1);
	frame_1->myglc_1->render();
}
