//-----------------------------------------------------------
//    main.cpp
//-----------------------------------------------------------
#include "main.h"
#include "myapp.h"
#include "v/myframe.h"
#include "v/myglcanvas.h"
#include "c/mycontroller.h"
#include "s/csound.h"
#include "m/shapemod/poly_line.h"
#include "m/shapemod/line.h"
#include "m/shapemod/dot.h"
#include "m/wavedata.h"
#include <wx/wx.h>
#include <wx/image.h>
//-----------------------------------------------------------
//    
//-----------------------------------------------------------
unsigned const int id_frame = 1000;
unsigned const int id_slider1 = 1001;
unsigned const int id_myglcanvas = 1004;
unsigned const int id_button = 1005;
unsigned const int id_textctrl = 1006;
unsigned const int id_popmenu_select1 = 1007;
unsigned const int id_popmenu_select2 = 1008;
const int slider1_size[2] = {-1, 200};
unsigned const int slider1_min = 0;
unsigned const int slider1_max = 100;
unsigned const int max_freq = 12000;
unsigned const int min_freq = 30;
unsigned const int canvas_size[2] = {400, 300};
unsigned const int grid_num1 = 8;
unsigned const int grid_num2 = 6;
unsigned int sound_vol = 10;
unsigned int sound_freq = 1000;

//-----------------------------------------------------------
//    prototype declaration of functions
//-----------------------------------------------------------
void connect_events(myController* ctr_1);
void init_poly_line(Poly_Line** pl1);
void init_wavedata(WaveData** wd1);
//-----------------------------------------------------------
//    IMPLEMENT_APP
//-----------------------------------------------------------
IMPLEMENT_APP(myApp)

//-----------------------------------------------------------
//    myApp::OnInit()
//-----------------------------------------------------------
bool myApp::OnInit()
{
	wxInitAllImageHandlers();
	myController *controller_1;
	Poly_Line *poly_line_1;
	WaveData* wavedata_1;
	init_wavedata(&wavedata_1);
	init_poly_line(&poly_line_1);
	
	cSound *csound_1 = new cSound(wavedata_1);
	myFrame* frame_1 = new myFrame(NULL, id_frame,
		wxEmptyString, wxPoint(100, 100),
		wxDefaultSize,
		long(wxDEFAULT_FRAME_STYLE
			&~(wxRESIZE_BORDER | wxRESIZE_BOX | wxMAXIMIZE_BOX)));
	frame_1->set_poly_line(poly_line_1);
	controller_1 = new myController(frame_1, csound_1,
		wavedata_1, poly_line_1);
	compose_wave_from_line(wavedata_1, poly_line_1);
	
	frame_1->SetEventHandler(controller_1);
	frame_1->myglc_1->SetEventHandler(controller_1);
	frame_1->text_ctrl_1->SetEventHandler(controller_1);
	connect_events(controller_1);
	
	SetTopWindow(frame_1);
	frame_1->Show();
	return true;
}

//-----------------------------------------------------------
//    connect_events()
//-----------------------------------------------------------
void connect_events(myController* ctr_1)
{
	//-----------------------------------------------------------
	//    controls
	//-----------------------------------------------------------
	ctr_1->Connect(id_slider1,
		wxEVT_COMMAND_SLIDER_UPDATED,
		wxScrollEventHandler(myController::OnSlider1),
		NULL);
	ctr_1->Connect(id_textctrl,
		wxEVT_CHAR,
		wxKeyEventHandler(myController::OnKey),
		NULL);
	ctr_1->Connect(id_button,
		wxEVT_COMMAND_TOGGLEBUTTON_CLICKED,
		wxCommandEventHandler(myController::OnButton),
		NULL);
	//-----------------------------------------------------------
	//    close
	//-----------------------------------------------------------
	ctr_1->Connect(id_frame, wxEVT_CLOSE_WINDOW,
		wxCloseEventHandler(myController::OnClose),
		NULL);
	//-----------------------------------------------------------
	//    canvas
	//-----------------------------------------------------------
	ctr_1->Connect(id_myglcanvas, wxID_ANY, wxEVT_PAINT,
		wxPaintEventHandler(myController::OnPaintCanvas),
		NULL);
	ctr_1->Connect(id_myglcanvas, wxEVT_LEFT_DOWN,
		wxMouseEventHandler(myController::OnMouseMove),
		NULL);
	ctr_1->Connect(id_myglcanvas, wxEVT_RIGHT_DOWN,
		wxMouseEventHandler(myController::OnMouseRight),
		NULL);
	ctr_1->Connect(id_myglcanvas, wxEVT_RIGHT_UP,
		wxMouseEventHandler(myController::OnMouseUP),
		NULL);
	ctr_1->Connect(id_myglcanvas, wxEVT_LEFT_UP,
		wxMouseEventHandler(myController::OnMouseUP),
		NULL);
	ctr_1->Connect(id_myglcanvas, wxEVT_MOTION,
		wxMouseEventHandler(myController::OnMouseMove),
		NULL);
	ctr_1->Connect(id_myglcanvas, wxEVT_LEAVE_WINDOW,
		wxMouseEventHandler(myController::OnLeaveCanvas),
		NULL);
	ctr_1->Connect(id_popmenu_select1, wxEVT_COMMAND_MENU_SELECTED,
		wxCommandEventHandler(myController::OnPopMenuSelect1),
		NULL);
	ctr_1->Connect(id_popmenu_select2, wxEVT_COMMAND_MENU_SELECTED,
		wxCommandEventHandler(myController::OnPopMenuSelect2),
		NULL);
}

//------------------------------------------------------------
//    init()
//------------------------------------------------------------
void init_poly_line(Poly_Line** pl1)
{
	std::vector<Dot> ds_1;
	Dot d1(0, 0);
	ds_1.assign(4, d1);
	ds_1[0].set_pos(0, 260);
	ds_1[1].set_pos(200, 260);
	ds_1[2].set_pos(200, 40);
	ds_1[3].set_pos(399, 40);
	(*pl1) = new Poly_Line(ds_1);
}

void init_wavedata(WaveData** wd1)
{
	(*wd1) = new WaveData;
	(*wd1)->sample_rate = 44100;
	(*wd1)->amplitude = double(sound_vol) * 0.01;
	(*wd1)->frames_per_buffer = 128;
	(*wd1)->table_size = 1000;
	(*wd1)->frequency = 1000;
	(*wd1)->value = new float[(*wd1)->table_size];
	(*wd1)->r_value = new float[(*wd1)->table_size];
	(*wd1)->dn = (double)(*wd1)->table_size * (double)(*wd1)->frequency
			/ (double)(*wd1)->sample_rate;
	for(int i=0; i<(*wd1)->table_size; i++ )
	{
		(*wd1)->r_value[i] = 0.0;
		(*wd1)->value[i] = 0.0;
	}
}

//------------------------------------------------------------
//    compose_wave_from_line
//------------------------------------------------------------
void compose_wave_from_line(WaveData* wd1, Poly_Line* pl1)
{
	if((wd1==0)||(pl1==0)) return;
	int i1, lnum, lsz = pl1->get_line_size();
	int tbsz = wd1->table_size;
	float wave1[tbsz];
	double amp = wd1->amplitude;
	double c1, px, py, dx = (double)canvas_size[0] / (double)tbsz;
	double l1 = 2.0 / (double)canvas_size[1];
	int x1, x2, y1, y2;
	lnum = 0;
	x1 = 0;
	for(i1 = 0; i1 < tbsz; i1++){
		px = dx * (double)i1;
		while(1){
			x1 = pl1->line(lnum)->get_dot_1()->get_pos_x();
			x2 = pl1->line(lnum)->get_dot_2()->get_pos_x();
			if(((px>=(double)x1)&&(px<=(double)x2))||(lnum>(lsz-2))) break;
			lnum++;
		}
		y1 = pl1->line(lnum)->get_dot_1()->get_pos_y();
		if((x2-x1)==0){
			wave1[i1] = (double)y1 * l1;
		}else{
			y2 = pl1->line(lnum)->get_dot_2()->get_pos_y();
			c1 = (double)(y2 - y1) / (double)(x2 - x1);
			py = (double)y1 + c1 * (px - (double)x1)
				- (double)canvas_size[1] * 0.5;
			py *= l1;
			wave1[i1] = py;
		}
	}
	
	for(i1 = 0; i1 < tbsz; i1++){
		if((wave1[i1]>=-1.0)&&( wave1[i1]<=1.0)){
			wd1->r_value[i1] = wave1[i1];
			wd1->value[i1] = wave1[i1] * amp;
		}
	}
}
