//------------------------------------------------------------
//    csound
//------------------------------------------------------------
#include "csound.h"
#include "../m/wavedata.h"
#include "../main.h"
#include <stdio.h>
#include <math.h>
#include <portaudio.h>
//-----------------------------------------------------------
//    constructor
//-----------------------------------------------------------
cSound::cSound(WaveData* wd1)
{
	wdata_1 = wd1;
	initialized = false;
	playing = false;
	stream = 0;
	init_pa();
}

//------------------------------------------------------------
//    destructor
//------------------------------------------------------------
cSound::~cSound()
{
}

//------------------------------------------------------------
//    callback function
//------------------------------------------------------------
static int cSoundCallback(const void *inputBuffer, void *outputBuffer,
	unsigned long framesPerBuffer,
	const PaStreamCallbackTimeInfo* timeInfo,
	PaStreamCallbackFlags statusFlags,
	void *userData );

//------------------------------------------------------------
//    init_pa
//------------------------------------------------------------
int cSound::init_pa()
{
	if(initialized) return -1;
	int er1;
	er1 = Pa_Initialize();
	wdata_1->left_phase = 0;
	wdata_1->right_phase = 0;
	outputParameters.device = Pa_GetDefaultOutputDevice();
	outputParameters.channelCount = 2;
	outputParameters.sampleFormat = paFloat32;
	outputParameters.suggestedLatency =
		Pa_GetDeviceInfo( outputParameters.device )->defaultLowOutputLatency;
	outputParameters.hostApiSpecificStreamInfo = NULL;
	int er2 = Pa_OpenStream(
		&stream,
		0,
		&outputParameters,
		wdata_1->sample_rate,
		wdata_1->frames_per_buffer,
		paClipOff,
		cSoundCallback,
		wdata_1 );
	if((er1!=-1)&&(er2!=-1)){
		initialized = true;
		return 1;
	}else{
		return -1;
	}
}

//------------------------------------------------------------
//    cSound::start_pa
//------------------------------------------------------------
int cSound::start_pa()
{
	int err;
	if(initialized==false) return -1;
	err = Pa_StartStream( stream );
	return err;
}

//------------------------------------------------------------
//    cSound::stop_pa
//------------------------------------------------------------
int cSound::stop_pa()
{
	int err;
	if((!initialized)&&(!playing)) return -1;
	err = Pa_StopStream( stream );
	return err;
}

//------------------------------------------------------------
//    cSound::close_pa
//------------------------------------------------------------
int cSound::close_pa()
{
	if(initialized==false) return -1;
	int err = Pa_CloseStream( stream );
	stream = 0;
	initialized = false;
	return err;
}

//------------------------------------------------------------
//    callback function
//------------------------------------------------------------
static int cSoundCallback(const void *inputBuffer, void *outputBuffer,
	unsigned long framesPerBuffer,
	const PaStreamCallbackTimeInfo* timeInfo,
	PaStreamCallbackFlags statusFlags,
	void *userData )
{
	WaveData *data = (WaveData*)userData;
	float *out = (float*)outputBuffer;
	unsigned long i;
	int finished = 0;
	(void) timeInfo;
	(void) statusFlags;
	(void) inputBuffer;
	for( i=0; i<framesPerBuffer; i++ )
	{
		*out++ = data->value[(unsigned int)data->left_phase];
		*out++ = data->value[(unsigned int)data->right_phase];
		data->left_phase += data->dn;
		if( data->left_phase >= data->table_size ) data->left_phase -= (double)data->table_size;
		data->right_phase += data->dn;
		if( data->right_phase >= data->table_size ) data->right_phase -= (double)data->table_size;
	}
	return finished;
}

